unit TFilo;

interface

uses
  Classes, SysUtils;

type
  TTFilo = class
  private
    FItems    : TList;
    FUseAsFifo: Boolean;
    function GetIsEmpty: Boolean;
    function GetCount:   Integer;
  public
    constructor Create;
    destructor  Destroy; override;
    //----------------------------------------
    // Procedures
    procedure Add(Value:  Integer);
    procedure Clear;
    //----------------------------------------
    // Functions
    function GetCurrent (Remove: Boolean): Integer;
    function RemoveAllOf(Entry:  Integer): Integer;
    //----------------------------------------
    // Properties
    property UseAsFifo: Boolean read FUseAsFifo write FUseAsFifo;
    property Count:     Integer read GetCount;
    property IsEmpty:   Boolean read GetIsEmpty;
  end;

implementation

//----------------------------------------
// Create the class
constructor TTFilo.Create;
  begin
  inherited Create;
  FItems     := TList.Create;
  FUseAsFifo := False;
  end;

//----------------------------------------
// Destroy the class
destructor TTFilo.Destroy;
  begin
  FItems.Free;
  inherited Destroy;
  end;

//----------------------------------------
// Get last entry entered
function TTFilo.GetCurrent(Remove: Boolean): Integer;
  begin
  if IsEmpty = True then Result := -1
  else
    begin
    if UseAsFifo = True then
      begin
      Result := Integer(FItems.First);
      if Remove = True then FItems.Delete(0);
      end
    else
      begin
      Result := Integer(FItems.Last);
      if Remove = True then FItems.Delete(FItems.Count -1 );
      end;
    end;
  end;

//----------------------------------------
// Add entry to list
procedure TTFilo.Add(Value: Integer);
  begin
  FItems.Add(Pointer(Value));
  end;

//----------------------------------------
// Removes all entries specified by "Entry"
function TTFilo.RemoveAllOf(Entry: Integer): Integer;
  begin
  Result := 0;
  while FItems.IndexOf(Pointer(Entry)) <> -1 do
    begin
    FItems.Delete(FItems.IndexOf(Pointer(Entry)));
    Inc(Result);
    end;
  end;

//----------------------------------------
// True if list is empty
function TTFilo.GetIsEmpty: Boolean;
  begin
  Result := (FItems.Count = 0);
  end;

//----------------------------------------
// Gets number of entries
function TTFilo.GetCount: Integer;
  begin
  Result := FItems.Count;
  end;

//----------------------------------------
// Clears the list
procedure TTFilo.Clear;
  begin
  FItems.Clear;
  end;

end.
